@@ -12,9 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
 import unittest
 
 import mock
+import requests
+from six.moves import http_client
 
 
 def _make_credentials():
@@ -23,6 +26,30 @@ def _make_credentials():
     return mock.Mock(spec=google.auth.credentials.Credentials)
 
 
+def _make_response(status=http_client.OK, content=b'', headers={}):
+    response = requests.Response()
+    response.status_code = status
+    response._content = content
+    response.headers = headers
+    response.request = requests.Request()
+    return response
+
+
+def _make_json_response(data, status=http_client.OK, headers=None):
+    headers = headers or {}
+    headers['Content-Type'] = 'application/json'
+    return _make_response(
+        status=status,
+        content=json.dumps(data).encode('utf-8'),
+        headers=headers)
+
+
+def _make_requests_session(responses):
+    session = mock.create_autospec(requests.Session, instance=True)
+    session.request.side_effect = responses
+    return session
+
+
 class TestClient(unittest.TestCase):
 
     @staticmethod
@@ -140,13 +167,15 @@ class TestClient(unittest.TestCase):
             'b',
             'nonesuch?projection=noAcl',
         ])
-        http = client._http_internal = _Http(
-            {'status': '404', 'content-type': 'application/json'},
-            b'{}',
-        )
-        self.assertRaises(NotFound, client.get_bucket, NONESUCH)
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertEqual(http._called_with['uri'], URI)
+        http = _make_requests_session([
+            _make_json_response({}, status=http_client.NOT_FOUND)])
+        client._http_internal = http
+
+        with self.assertRaises(NotFound):
+            client.get_bucket(NONESUCH)
+
+        http.request.assert_called_once_with(
+            method='GET', url=URI, data=mock.ANY, headers=mock.ANY)
 
     def test_get_bucket_hit(self):
         from google.cloud.storage.bucket import Bucket
@@ -163,16 +192,17 @@ class TestClient(unittest.TestCase):
             'b',
             '%s?projection=noAcl' % (BLOB_NAME,),
         ])
-        http = client._http_internal = _Http(
-            {'status': '200', 'content-type': 'application/json'},
-            '{{"name": "{0}"}}'.format(BLOB_NAME).encode('utf-8'),
-        )
+
+        data = {'name': BLOB_NAME}
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
 
         bucket = client.get_bucket(BLOB_NAME)
+
         self.assertIsInstance(bucket, Bucket)
         self.assertEqual(bucket.name, BLOB_NAME)
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertEqual(http._called_with['uri'], URI)
+        http.request.assert_called_once_with(
+            method='GET', url=URI, data=mock.ANY, headers=mock.ANY)
 
     def test_lookup_bucket_miss(self):
         PROJECT = 'PROJECT'
@@ -187,14 +217,15 @@ class TestClient(unittest.TestCase):
             'b',
             'nonesuch?projection=noAcl',
         ])
-        http = client._http_internal = _Http(
-            {'status': '404', 'content-type': 'application/json'},
-            b'{}',
-        )
+        http = _make_requests_session([
+            _make_json_response({}, status=http_client.NOT_FOUND)])
+        client._http_internal = http
+
         bucket = client.lookup_bucket(NONESUCH)
+
         self.assertIsNone(bucket)
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertEqual(http._called_with['uri'], URI)
+        http.request.assert_called_once_with(
+            method='GET', url=URI, data=mock.ANY, headers=mock.ANY)
 
     def test_lookup_bucket_hit(self):
         from google.cloud.storage.bucket import Bucket
@@ -211,16 +242,16 @@ class TestClient(unittest.TestCase):
             'b',
             '%s?projection=noAcl' % (BLOB_NAME,),
         ])
-        http = client._http_internal = _Http(
-            {'status': '200', 'content-type': 'application/json'},
-            '{{"name": "{0}"}}'.format(BLOB_NAME).encode('utf-8'),
-        )
+        data = {'name': BLOB_NAME}
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
 
         bucket = client.lookup_bucket(BLOB_NAME)
+
         self.assertIsInstance(bucket, Bucket)
         self.assertEqual(bucket.name, BLOB_NAME)
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertEqual(http._called_with['uri'], URI)
+        http.request.assert_called_once_with(
+            method='GET', url=URI, data=mock.ANY, headers=mock.ANY)
 
     def test_create_bucket_conflict(self):
         from google.cloud.exceptions import Conflict
@@ -236,14 +267,14 @@ class TestClient(unittest.TestCase):
             client._connection.API_VERSION,
             'b?project=%s' % (PROJECT,),
         ])
-        http = client._http_internal = _Http(
-            {'status': '409', 'content-type': 'application/json'},
-            '{"error": {"message": "Conflict"}}',
-        )
+        data = {'error': {'message': 'Conflict'}}
+        http = _make_requests_session([
+            _make_json_response(data, status=http_client.CONFLICT)])
+        client._http_internal = http
 
         self.assertRaises(Conflict, client.create_bucket, BLOB_NAME)
-        self.assertEqual(http._called_with['method'], 'POST')
-        self.assertEqual(http._called_with['uri'], URI)
+        http.request.assert_called_once_with(
+            method='POST', url=URI, data=mock.ANY, headers=mock.ANY)
 
     def test_create_bucket_success(self):
         from google.cloud.storage.bucket import Bucket
@@ -259,16 +290,16 @@ class TestClient(unittest.TestCase):
             client._connection.API_VERSION,
             'b?project=%s' % (PROJECT,),
         ])
-        http = client._http_internal = _Http(
-            {'status': '200', 'content-type': 'application/json'},
-            '{{"name": "{0}"}}'.format(BLOB_NAME).encode('utf-8'),
-        )
+        data = {'name': BLOB_NAME}
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
 
         bucket = client.create_bucket(BLOB_NAME)
+
         self.assertIsInstance(bucket, Bucket)
         self.assertEqual(bucket.name, BLOB_NAME)
-        self.assertEqual(http._called_with['method'], 'POST')
-        self.assertEqual(http._called_with['uri'], URI)
+        http.request.assert_called_once_with(
+            method='POST', url=URI, data=mock.ANY, headers=mock.ANY)
 
     def test_list_buckets_empty(self):
         from six.moves.urllib.parse import parse_qs
@@ -278,59 +309,50 @@ class TestClient(unittest.TestCase):
         CREDENTIALS = _make_credentials()
         client = self._make_one(project=PROJECT, credentials=CREDENTIALS)
 
-        EXPECTED_QUERY = {
-            'project': [PROJECT],
-            'projection': ['noAcl'],
-        }
-        http = client._http_internal = _Http(
-            {'status': '200', 'content-type': 'application/json'},
-            b'{}',
-        )
-        buckets = list(client.list_buckets())
-        self.assertEqual(len(buckets), 0)
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertIsNone(http._called_with['body'])
+        http = _make_requests_session([_make_json_response({})])
+        client._http_internal = http
 
-        BASE_URI = '/'.join([
+        buckets = list(client.list_buckets())
+
+        self.assertEqual(len(buckets), 0)
+
+        http.request.assert_called_once_with(
+            method='GET', url=mock.ANY, data=mock.ANY, headers=mock.ANY)
+
+        requested_url = http.request.mock_calls[0][2]['url']
+        expected_base_url = '/'.join([
             client._connection.API_BASE_URL,
             'storage',
             client._connection.API_VERSION,
             'b',
         ])
-        URI = http._called_with['uri']
-        self.assertTrue(URI.startswith(BASE_URI))
-        uri_parts = urlparse(URI)
-        self.assertEqual(parse_qs(uri_parts.query), EXPECTED_QUERY)
+        self.assertTrue(requested_url.startswith(expected_base_url))
+
+        expected_query = {
+            'project': [PROJECT],
+            'projection': ['noAcl'],
+        }
+        uri_parts = urlparse(requested_url)
+        self.assertEqual(parse_qs(uri_parts.query), expected_query)
 
     def test_list_buckets_non_empty(self):
-        from six.moves.urllib.parse import parse_qs
-        from six.moves.urllib.parse import urlencode
-        from six.moves.urllib.parse import urlparse
-
         PROJECT = 'PROJECT'
         CREDENTIALS = _make_credentials()
         client = self._make_one(project=PROJECT, credentials=CREDENTIALS)
 
         BUCKET_NAME = 'bucket-name'
-        query_params = urlencode({'project': PROJECT, 'projection': 'noAcl'})
-        BASE_URI = '/'.join([
-            client._connection.API_BASE_URL,
-            'storage',
-            client._connection.API_VERSION,
-        ])
-        URI = '/'.join([BASE_URI, 'b?%s' % (query_params,)])
-        http = client._http_internal = _Http(
-            {'status': '200', 'content-type': 'application/json'},
-            '{{"items": [{{"name": "{0}"}}]}}'.format(BUCKET_NAME)
-            .encode('utf-8'),
-        )
+
+        data = {'items': [{'name': BUCKET_NAME}]}
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
+
         buckets = list(client.list_buckets())
+
         self.assertEqual(len(buckets), 1)
         self.assertEqual(buckets[0].name, BUCKET_NAME)
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertTrue(http._called_with['uri'].startswith(BASE_URI))
-        self.assertEqual(parse_qs(urlparse(http._called_with['uri']).query),
-                         parse_qs(urlparse(URI).query))
+
+        http.request.assert_called_once_with(
+            method='GET', url=mock.ANY, data=mock.ANY, headers=mock.ANY)
 
     def test_list_buckets_all_arguments(self):
         from six.moves.urllib.parse import parse_qs
@@ -345,19 +367,10 @@ class TestClient(unittest.TestCase):
         PREFIX = 'subfolder'
         PROJECTION = 'full'
         FIELDS = 'items/id,nextPageToken'
-        EXPECTED_QUERY = {
-            'project': [PROJECT],
-            'maxResults': [str(MAX_RESULTS)],
-            'pageToken': [PAGE_TOKEN],
-            'prefix': [PREFIX],
-            'projection': [PROJECTION],
-            'fields': [FIELDS],
-        }
 
-        http = client._http_internal = _Http(
-            {'status': '200', 'content-type': 'application/json'},
-            '{"items": []}',
-        )
+        data = {'items': []}
+        http = _make_requests_session([_make_json_response(data)])
+        client._http_internal = http
         iterator = client.list_buckets(
             max_results=MAX_RESULTS,
             page_token=PAGE_TOKEN,
@@ -367,19 +380,28 @@ class TestClient(unittest.TestCase):
         )
         buckets = list(iterator)
         self.assertEqual(buckets, [])
-        self.assertEqual(http._called_with['method'], 'GET')
-        self.assertIsNone(http._called_with['body'])
+        http.request.assert_called_once_with(
+            method='GET', url=mock.ANY, data=mock.ANY, headers=mock.ANY)
 
-        BASE_URI = '/'.join([
+        requested_url = http.request.mock_calls[0][2]['url']
+        expected_base_url = '/'.join([
             client._connection.API_BASE_URL,
             'storage',
             client._connection.API_VERSION,
-            'b'
+            'b',
         ])
-        URI = http._called_with['uri']
-        self.assertTrue(URI.startswith(BASE_URI))
-        uri_parts = urlparse(URI)
-        self.assertEqual(parse_qs(uri_parts.query), EXPECTED_QUERY)
+        self.assertTrue(requested_url.startswith(expected_base_url))
+
+        expected_query = {
+            'project': [PROJECT],
+            'maxResults': [str(MAX_RESULTS)],
+            'pageToken': [PAGE_TOKEN],
+            'prefix': [PREFIX],
+            'projection': [PROJECTION],
+            'fields': [FIELDS],
+        }
+        uri_parts = urlparse(requested_url)
+        self.assertEqual(parse_qs(uri_parts.query), expected_query)
 
     def test_page_empty_response(self):
         from google.cloud.iterator import Page
@@ -415,18 +437,3 @@ class TestClient(unittest.TestCase):
         self.assertEqual(page.remaining, 0)
         self.assertIsInstance(bucket, Bucket)
         self.assertEqual(bucket.name, blob_name)
-
-
-class _Http(object):
-
-    _called_with = None
-
-    def __init__(self, headers, content):
-        from httplib2 import Response
-
-        self._response = Response(headers)
-        self._content = content
-
-    def request(self, **kw):
-        self._called_with = kw
-        return self._response, self._content
